#include "GetSignal.h"

/************************************************************************/
/* Function: GetIndexKey						*/
/* Purpose: Input indices.txt and the file with selected indices list	*/
/*	    Output a 2D [N]x[20] array of indices			*/
/************************************************************************/
double **GetIndexKey(char *filename,int *index_num)
{
	int i,j,id=0;
	int lines,all_lines;
	char tline[CHARSIZE],*token,**index_id,key_id[CHARSIZE],string[1000];
	char seps[]   = " ,\t\n";
	int indicator[504],num_key=0;
	FILE *fid;
	double **index,tmp_index,**all_index;


	/***Read in all indices *****/
	if((fid=fopen("indices.txt","rt"))==NULL)
	{ printf("Error : cannot open indices.txt !\n"); exit(0); }

	all_lines=num_lines(fid);
        if((all_index=(double **)calloc(all_lines,sizeof(double *)))==NULL)
        { printf("Error : calloc to all_index \n"); exit(0); }
        if((index_id=(char **)calloc(all_lines,sizeof(char *)))==NULL)
        { printf("Error : calloc to index_id \n"); exit(0); }
	

	for (i=0;i<all_lines;i++)
	{
	        if((all_index[i]=(double *)calloc(AA_COUNT,sizeof(double)))==NULL)
        	{ printf("Error : calloc to all_index[] \n"); exit(0); }
	        if((index_id[i]=(char *)calloc(ID_LEN+1,sizeof(char)))==NULL)
        	{ printf("Error : calloc to index_id[] \n"); exit(0); }

		fscanf(fid,"%s",&string);
		for (j=0;j<ID_LEN;j++)	index_id[i][j]=string[j];
		index_id[i][ID_LEN]='\0';

		for (j=0;j<AA_COUNT;j++)
		{
			fscanf(fid,"%lf",&all_index[i][j]);
		}
	}

	fclose(fid);	

	/*********************read in wanted index from file****************/
	if((fid=fopen(filename,"rt"))==NULL)
	{ printf("Error : cannot open %s !\n",filename); exit(0); }
	

	lines=num_lines(fid);
	*index_num=lines;
	if((index=(double **)calloc(lines,sizeof(double *)))==NULL)
	{ printf("Error : calloc to index \n"); exit(0); }
	for (i=0;i<lines;i++)
		if((index[i]=(double *)calloc(AA_COUNT,sizeof(double)))==NULL)
		{ printf("Error : calloc to index[] \n"); exit(0); }
	
	id=0;

	while (fgets(tline,CHARSIZE,fid))
	{
		if ((sscanf(tline,"%s",&key_id))==1)
		{
			for (j=0;j<all_lines;j++)
			{
				if (strcmp(key_id,index_id[j])==0)
				{
					for (i=0;i<AA_COUNT;i++)
						index[id][i]=all_index[j][i];
					id+=1;
					break;
				}
			}
		}
	}

	fclose(fid);

	for (i=0;i<all_lines;i++)	free(all_index[i]);	free(all_index);
        for (i=0;i<all_lines;i++)       free(index_id[i]);     free(index_id);		

	return index;				
}

/************************************************************************/
/* Function: GetSignal							*/
/* Purpose: Input seq and output signal for each index num		*/
/* (Direct conversion from aa to signal)				*/
/************************************************************************/
double **GetSignal(char *seq,double **a,int index_num)
{
	int i,j;
	int seq_len;
	double **seq_sig,signal;
	double average=0.;
	
	seq_len=strlen(seq);
	
	if((seq_sig=(double **)calloc(index_num,sizeof(double *)))==NULL)
	{ printf("Error : calloc to seq_sig[3] \n"); exit(0); }
	
	for (i=0;i<index_num;i++)
	{
		for (j=0;j<20;j++)	average+=a[i][j];
		average/=20;
		if((seq_sig[i]=(double *)calloc(seq_len,sizeof(double)))==NULL)
		{ printf("Error : calloc to seq_sig[3][seq_len] \n"); exit(0); }
		
		for (j=0;j<seq_len;j++)
		{
			switch (toupper(seq[j]))
			{
		        	case 'A':
	        			signal =  a[i][0];
					break;
		        	case 'R':
        				signal =  a[i][1];
					break;
		        	case 'N':
        			     	signal =  a[i][2];
					break;
		        	case 'D':
        			     	signal =  a[i][3];
					break;
		        	case 'C':
        			     	signal =  a[i][4];
					break;
		        	case 'Q':
        			     	signal =  a[i][5];
					break;
		        	case 'E':
        			     	signal =  a[i][6];
					break;
		        	case 'G':
        			     	signal =  a[i][7];
					break;
		        	case 'H':
        			     	signal =  a[i][8];
					break;
		        	case 'I':
        			     	signal =  a[i][9];
					break;
		           	case 'L':
	        		    	signal =  a[i][10];
					break;
		        	case 'K':
        			     	signal =  a[i][11];
					break;
		        	case 'M':
        			     	signal =  a[i][12];
					break;
		        	case 'F':
		        	     	signal =  a[i][13];
					break;
		        	case 'P':
		        	     	signal =  a[i][14];
					break;
		        	case 'S':
		        	     	signal =  a[i][15];
					break;
		        	case 'T':
        			     	signal =  a[i][16];
					break;
		        	case 'W':
		        	     	signal =  a[i][17];
					break;
		        	case 'Y':
		        	     	signal =  a[i][18];
					break;
		        	case 'V':
		        	     	signal =  a[i][19];
					break;
				default:
					signal = average;
				
			}//case
			seq_sig[i][j]=signal;
		} //j loop
	}// i loop
	
	return seq_sig;
}

/************************************************************************/
/* Function: SigWin							*/
/* Purpose: Input 2D seq_sig and output windowed signal			*/
/************************************************************************/
double **SigWin(double **seq_sig,int seq_len,int index_num)
{
	double **seq_win;
	int i,j,k,half_win,range_low,range_up;

	half_win=(WIN-1)/2;

	if((seq_win=(double **)calloc(index_num,sizeof(double *)))==NULL)
	{ printf("Error : calloc to seq_win[3] \n"); exit(0); }
	
	for (i=0;i<index_num;i++)
	{
		if((seq_win[i]=(double *)calloc(seq_len,sizeof(double)))==NULL)
		{ printf("Error : calloc to seq_win[3][seq_len] \n"); exit(0); }
		
		for (j=0;j<half_win;j++)
		{
			seq_win[i][j]=0.;
			range_up=max(half_win,2*(j+1)-1);
			
			for (k=0;k<range_up;k++)
				seq_win[i][j]+=seq_sig[i][k];
			seq_win[i][j]/=range_up;
		}

		for (j=half_win;j<seq_len-half_win;j++)
		{
			seq_win[i][j]=0.;
			for (k=j-half_win;k<j+half_win+1;k++)			
				seq_win[i][j]+=seq_sig[i][k];
			seq_win[i][j]/=WIN;
		}

		for (j=seq_len-half_win;j<seq_len;j++)
		{
			seq_win[i][j]=0.;
			range_low=min(seq_len-half_win-1,2*j-seq_len+1);
			for (k=range_low;k<seq_len;k++)			
				seq_win[i][j]+=seq_sig[i][k];
			seq_win[i][j]/=(seq_len-range_low);
		}

		
	}
	
	return seq_win;	
}

/************************************************************************/
/* Function: GetWave							*/
/* Purpose: Get level 2 waveform				*/
/************************************************************************/
double **GetWave(double **win,int seq_len,int index_num,char type)
{
	int i,j;
	int level,ll,ld;
	double **wave;
	WAVETYPE rwave;
	WAVE wave_coef;

	level=LEVEL;
	if ((wave=(double **)calloc(index_num,sizeof(double *)))==NULL)
	{printf("Error: calloc to wave\n"); exit(0);}

	for (i=0;i<index_num;i++)
	{
		wave_coef=wavedec(win[i],level,seq_len,&ld,&ll);
		rwave=approximation(wave_coef,ld,ll,level,seq_len);
		if (type=='a')
			wave[i]=zscore(rwave.approximations[level-1],seq_len);
		else if (type=='d')
			wave[i]=zscore(rwave.details[level-1],seq_len);
		else
		{printf("Error(Domain_Process:GetWave): wrong type\n"); exit(0);}
		for (j=0;j<level;j++)	{free(rwave.approximations[j]);	free(rwave.details[j]);}
		free(wave_coef.details);	free(wave_coef.length);
	}
	return wave;
}

/************************************************************************/
/* Function: SigWinSingle							*/
/* Purpose: Input 1D signal and output windowed signal			*/
/************************************************************************/
double *SigWinSingle(double *seq_sig,int seq_len)
{
	double *seq_win;
	int j,k,half_win,range_low,range_up;

	half_win=(WIN-1)/2;

	
		if((seq_win=(double *)calloc(seq_len,sizeof(double)))==NULL)
		{ printf("Error : calloc to seq_win[3][seq_len] \n"); exit(0); }
		
		for (j=0;j<half_win;j++)
		{
			seq_win[j]=0.;
			range_up=max(half_win,2*(j+1)-1);
			
			for (k=0;k<range_up;k++)
				seq_win[j]+=seq_sig[k];
			seq_win[j]/=range_up;
		}

		for (j=half_win;j<seq_len-half_win;j++)
		{
			seq_win[j]=0.;
			for (k=j-half_win;k<j+half_win+1;k++)			
				seq_win[j]+=seq_sig[k];
			seq_win[j]/=WIN;
		}

		for (j=seq_len-half_win;j<seq_len;j++)
		{
			seq_win[j]=0.;
			range_low=min(seq_len-half_win-1,2*j-seq_len+1);
			for (k=range_low;k<seq_len;k++)			
				seq_win[j]+=seq_sig[k];
			seq_win[j]/=(seq_len-range_low);
		}

		
	
	
	return seq_win;	
}

/************************************************************************/
/* Function: GetSlope							*/
/* Purpose: Get slope for ddp Euclidean distance calculation		*/
/************************************************************************/
double **GetSlope(double **wave,int seq_len,int index_num)
{
	int i,j;
	double **slope;

	if ((slope=(double **)calloc(index_num,sizeof(double *)))==NULL)
	{printf("Error: calloc to slope\n"); exit(0);}

	for (i=0;i<index_num;i++)
	{
		slope[i][0]=((wave[i][1]-wave[i][0])+(wave[i][2]-wave[i][0])/2.)/2.;
		for (j=1;j<seq_len-1;j++)
		{
			slope[i][j]=((wave[i][j+1]-wave[i][j])+(wave[i][j+2]-wave[i][j])/2.)/2.;
		}
		slope[i][seq_len-1]=((wave[i][seq_len-2]-wave[i][seq_len-3])+(wave[i][seq_len-1]-wave[i][seq_len-3])/2.)/2.;
	}

	return slope;
}


/************************************************************************/
/* Function: GetWaveSingle							*/
/* Purpose: Get level 2 waveform				*/
/************************************************************************/
double *GetWaveSingle(double *win,int seq_len)
{
	int j;
	int level,ll,ld;
	double *wave;
	WAVETYPE rwave;
	WAVE wave_coef;

	level=LEVEL;
	wave_coef=wavedec(win,level,seq_len,&ld,&ll);
	rwave=approximation(wave_coef,ld,ll,level,seq_len);
	wave=zscore(rwave.approximations[level-1],seq_len);
	for (j=0;j<level;j++)	{free(rwave.approximations[j]);	free(rwave.details[j]);}
	free(wave_coef.details);	free(wave_coef.length);

	return wave;
}



/************************************************************************/
/* Function: zscore							*/
/* Purpose: normalization						*/
/************************************************************************/
double *zscore(double *input,int len)
{
	double sd=0.,avg=0.;
	double *output;
	int i;

	if((output=(double *)calloc(len,sizeof(double)))==NULL)
	{ printf("Error : calloc to output[len]\n"); exit(0); }

	for (i=0;i<len;i++)
		avg+=input[i];
	avg/=len;

	for (i=0;i<len;i++)
		sd+=(input[i]-avg)*(input[i]-avg);
	sd=sqrt(sd/(len-1));

	for (i=0;i<len;i++)
	{
		output[i]=(input[i]-avg)/sd;
		//output[i]=input[i];
	}

	return output;
}

/************************************************************************/
/* Function: wavedec							*/
/* Purpose: Wavelet destruction						*/
/************************************************************************/
WAVE wavedec(double *x, int n,int ssx,int *ld, int *ll)
{
	int sx,sd_new,sd_old,sd;
	double *x_new,*x_old,*detail,*keep_detail;
	int *slen,*len,len_count,i,j;
	WAVE wave_coef;
	DWTTYPE dwt_coef;

	Lo_D[0]=(double)-0.0000;		Lo_D[1]=(double)0.0001;		Lo_D[2]=(double)-0.0001;		Lo_D[3]=(double)-0.0007;		Lo_D[4]=(double)0.0020;    
	Lo_D[5]=(double)0.0014;		Lo_D[6]=(double)-0.0107;    	Lo_D[7]=(double)0.0036;    	Lo_D[8]=(double)0.0332;   	Lo_D[9]=(double)-0.0295;
	Lo_D[10]=(double)-0.0714;	Lo_D[11]=(double)0.0931;    	Lo_D[12]=(double)0.1274;   	Lo_D[13]=(double)-0.1959;   	Lo_D[14]=(double)-0.2498;    
	Lo_D[15]=(double)0.2812;    	Lo_D[16]=(double)0.6885;    	Lo_D[17]=(double)0.5272;    	Lo_D[18]=(double)0.1882;    	Lo_D[19]=(double)0.0267;

	Hi_D[0]=(double)-0.0267;    	Hi_D[1]=(double)0.1882;   	Hi_D[2]=(double)-0.5272;    	Hi_D[3]=(double)0.6885;   	Hi_D[4]=(double)-0.2812;   
	Hi_D[5]=(double)-0.2498;    	Hi_D[6]=(double)0.1959;   	Hi_D[7]=(double)0.1274;   	Hi_D[8]=(double)-0.0931;   	Hi_D[9]=(double)-0.0714;
	Hi_D[10]=(double)0.0295;    	Hi_D[11]=(double)0.0332;   	Hi_D[12]=(double)-0.0036;   	Hi_D[13]=(double)-0.0107;  	Hi_D[14]=(double)-0.0014;
	Hi_D[15]=(double)0.0020;    	Hi_D[16]=(double)0.0007;   	Hi_D[17]=(double)-0.0001;   	Hi_D[18]=(double)-0.0001;   	Hi_D[19]=(double)-0.0000;


	sx=ssx;
	if((x_new=(double *)calloc(sx+1,sizeof(double)))==NULL)
	{ printf("Error (wavedec) : calloc to x_new[] \n"); exit(0); }

	if((len=(int *)calloc(1,sizeof(int)))==NULL)
	{ printf("Error (wavedec) : calloc to len[] \n"); exit(0); }
	len[0]=sx;
	len_count=1;
	//x_new=x;
	for (i=0;i<sx;i++)		x_new[i]=x[i];
	sd_old=0;
	if((detail=(double *)calloc(sd_old,sizeof(double)))==NULL)
	{ printf("Error (wavedec) : calloc to keep_detail[] \n"); exit(0); }
	
	for (i=0;i<n;i++)
	{
		if((x_old=(double *)calloc(sx,sizeof(double)))==NULL)
		{ printf("Error (wavedec) : calloc to x_old[] \n"); exit(0); }
		//x_old=x_new;
		
		for (j=0;j<sx;j++)	x_old[j]=x_new[j];
		free(x_new);
		
		
		//x_new,detail=dwt(x_old);
		dwt_coef=dwt(x_old,&sx,&sd);
		
		/*for (j=0;j<sx;j++)
			printf("[%d][%d]=%f\n",i,j,dwt_coef.approximation[j]);*/
		free(x_old);
		
		if((x_new=(double *)calloc(sx,sizeof(double)))==NULL)
		{ printf("Error (wavedec) : calloc to x_new[] \n"); exit(0); }
		for(j=0;j<sx;j++)	x_new[j]=dwt_coef.approximation[j];
		//keep_detail=[detail keep_detail]
		sd_new=sd;
		if((keep_detail=(double *)calloc(sd_new+sd_old,sizeof(double)))==NULL)
		{ printf("Error (wavedec) : calloc to keep_detail[] \n"); exit(0); }
		for (j=0;j<sd_new;j++)			keep_detail[j]=dwt_coef.details[j];
		for (j=sd_new;j<sd_new+sd_old;j++)			keep_detail[j]=detail[j-sd_new];
		free(detail);
		if((detail=(double *)calloc(sd_new+sd_old,sizeof(double)))==NULL)
		{ printf("Error (wavedec) : calloc to keep_detail[] \n"); exit(0); }
		for (j=0;j<sd_new+sd_old;j++)	detail[j]=keep_detail[j];
		free(keep_detail);

		len_count+=1;
		if((slen=(int *)calloc(len_count,sizeof(int)))==NULL)
		{ printf("Error (wavedec) : calloc to slen[] \n"); exit(0); }
		slen[0]=sd_new;
		for (j=1;j<len_count;j++)	slen[j]=len[j-1];
		free(len);
		if((len=(int *)calloc(len_count,sizeof(int)))==NULL)
		{ printf("Error (wavedec) : calloc to len[] \n"); exit(0); }
		for (j=0;j<len_count;j++)	len[j]=slen[j];
		free(slen);
		sd_old=sd_new+sd_old;

	}

	sd_new=sx;
	if((keep_detail=(double *)calloc(sd_new+sd_old,sizeof(double)))==NULL)
	{ printf("Error (wavedec) : calloc to keep_detail[] \n"); exit(0); }
	for (j=0;j<sd_new;j++)			keep_detail[j]=dwt_coef.approximation[j];
	for (j=sd_new;j<sd_new+sd_old;j++)			keep_detail[j]=detail[j-sd_new];
	free(detail);
	if((detail=(double *)calloc(sd_new+sd_old,sizeof(double)))==NULL)
	{ printf("Error (wavedec) : calloc to keep_detail[] \n"); exit(0); }
	for (j=0;j<sd_new+sd_old;j++)	detail[j]=keep_detail[j];
	free(keep_detail);
	

	len_count+=1;
	if((slen=(int *)calloc(len_count,sizeof(int)))==NULL)
	{ printf("Error (wavedec) : calloc to slen[] \n"); exit(0); }
	slen[0]=sd_new;
	for (j=1;j<len_count;j++)	slen[j]=len[j-1];
	free(len);
	if((len=(int *)calloc(len_count,sizeof(int)))==NULL)
	{ printf("Error (wavedec) : calloc to len[] \n"); exit(0); }
	for (j=0;j<len_count;j++)	len[j]=slen[j];
	free(slen);
	sd_old=sd_new+sd_old;

	free(x_new);


	if((wave_coef.details=(double *)calloc(sd_old,sizeof(double)))==NULL)
	{ printf("Error (wavedec) : calloc to len[] \n"); exit(0); }
	if((wave_coef.length=(int *)calloc(len_count,sizeof(int)))==NULL)
	{ printf("Error (wavedec) : calloc to len[] \n"); exit(0); }

	for (i=0;i<sd_old;i++)
		wave_coef.details[i]=detail[i];
	for (i=0;i<len_count;i++)
		wave_coef.length[i]=len[i];
	*ll=len_count;
	*ld=sd_old;
	free(detail);	free(len);

	return wave_coef;

}
/************************************************************************/
/* All the following functions are for wavelet destructuion		*/
/************************************************************************/
WAVETYPE approximation(WAVE wave_coef,int ld, int ll,int n,int sx)
{
	int i,j;
	double *coef,**x,**y,*x_tmp,*y_tmp;
	int *len;
	WAVETYPE rwave;


	Lo_R[0]=(double)0.0267;    Lo_R[1]=(double)0.1882;    Lo_R[2]=(double)0.5272;		Lo_R[3]=(double)0.6885;		Lo_R[4]=(double)0.2812;
	Lo_R[5]=(double)-0.2498;   Lo_R[6]=(double)-0.1959;   Lo_R[7]=(double)0.1274;		Lo_R[8]=(double)0.0931;		Lo_R[9]=(double)-0.0714;   
	Lo_R[10]=(double)-0.0295;  Lo_R[11]=(double)0.0332;  Lo_R[12]=(double)0.0036;		Lo_R[13]=(double)-0.0107;	Lo_R[14]=(double)0.0014;
	Lo_R[15]=(double)0.0020;   Lo_R[16]=(double)-0.0007;  Lo_R[17]=(double)-0.0001;	Lo_R[18]=(double)0.0001;		Lo_R[19]=(double)-0.0000;


	Hi_R[0]=(double)-0.0000;   Hi_R[1]=(double)-0.0001;   Hi_R[2]=(double)-0.0001;		Hi_R[3]=(double)0.0007;		Hi_R[4]=(double)0.0020;
	Hi_R[5]=(double)-0.0014;   Hi_R[6]=(double)-0.0107;   Hi_R[7]=(double)-0.0036;		Hi_R[8]=(double)0.0332;		Hi_R[9]=(double)0.0295;   
	Hi_R[10]=(double)-0.0714;  Hi_R[11]=(double)-0.0931; Hi_R[12]=(double)0.1274;		Hi_R[13]=(double)0.1959;		Hi_R[14]=(double)-0.2498;
	Hi_R[15]=(double)-0.2812;  Hi_R[16]=(double)0.6885;   Hi_R[17]=(double)-0.5272;	Hi_R[18]=(double)0.1882;		Hi_R[19]=(double)-0.0267;
	
	if((coef=(double *)calloc(ld,sizeof(double)))==NULL)
	{ printf("Error (approximation) : calloc to coef[] \n"); exit(0); }
	if((len=(int *)calloc(ll,sizeof(int)))==NULL)
	{ printf("Error (approximation) : calloc to len[] \n"); exit(0); }

	for (i=0;i<ld;i++)		coef[i]=wave_coef.details[i];
	for (i=0;i<ll;i++)		len[i]=wave_coef.length[i];
	
	if((x=(double **)calloc(n,sizeof(double *)))==NULL)
	{ printf("Error (appcoef) : calloc to x[] \n"); exit(0); }

	if((y=(double **)calloc(n,sizeof(double *)))==NULL)
	{ printf("Error (appcoef) : calloc to y[] \n"); exit(0); }
	
	for (i=1;i<=n;i++)
	{
		//approximation wrcoef=x
		x_tmp=wrcoef_a(coef,len,ll,i);
		
		if((x[i-1]=(double *)calloc(sx,sizeof(double)))==NULL)
		{ printf("Error (appcoef) : calloc to x[][] \n"); exit(0); }
		for(j=0;j<sx;j++)	x[i-1][j]=x_tmp[j];
		free(x_tmp);

		//details=y

		y_tmp=wrcoef_d(coef,len,ll,i);
		
		if((y[i-1]=(double *)calloc(sx,sizeof(double)))==NULL)
		{ printf("Error (appcoef) : calloc to y[][] \n"); exit(0); }
		for(j=0;j<sx;j++)	y[i-1][j]=y_tmp[j];
		free(y_tmp);
	
	}

	if((rwave.approximations=(double **)calloc(n,sizeof(double *)))==NULL)
	{ printf("Error (appcoef) : calloc to rwave.approximations[] \n"); exit(0); }
	if((rwave.details=(double **)calloc(n,sizeof(double *)))==NULL)
	{ printf("Error (appcoef) : calloc to rwave.details[] \n"); exit(0); }
	for (i=0;i<n;i++)
	{
		if((rwave.approximations[i]=(double *)calloc(sx,sizeof(double)))==NULL)
		{ printf("Error (appcoef) : calloc to rwave.approximations[] \n"); exit(0); }
		if((rwave.details[i]=(double *)calloc(sx,sizeof(double)))==NULL)
		{ printf("Error (appcoef) : calloc to rwave.details[] \n"); exit(0); }
		for (j=0;j<sx;j++)	rwave.approximations[i][j]=x[i][j];
		for (j=0;j<sx;j++)	rwave.details[i][j]=y[i][j];
		free(x[i]);	free(y[i]);
	}

	free(x);	free(y);

	return rwave;
	
}

double *wrcoef_d(double *coef,int *len,int ll,int i)
{
	double *dx,*y,*y_new;
	int sdx,rmax,nmax,nmin,imin,sy_new,j,k;

	rmax=ll;
	nmax=rmax-2;
	nmin=1;

	dx=detcoef(coef,len,ll,i,&sdx);
	
	imin=rmax-i;
	
	y=upsaconv(dx,Hi_R,len[imin],&sy_new,sdx);
	
	for (j=2;j<=i;j++)
	{
		if((y_new=(double *)calloc(sy_new,sizeof(double)))==NULL)
		{ printf("Error (appcoef) : calloc to y_new[] \n"); exit(0); }
		for(k=0;k<sy_new;k++)	y_new[k]=y[k];
		free(y);
		
		y = upsaconv(y_new,Lo_R,len[imin+j-1],&sy_new,sy_new);
		
		free(y_new);

	}
	
	return y;
}

double *wrcoef_a(double *coef,int *len,int ll,int i)
{
	double *x,*x_new;
	int j,k,sx_new,rmax,imin;

	x=appcoef(coef,len,ll,&sx_new,i);
	
	if((x_new=(double *)calloc(sx_new,sizeof(double)))==NULL)
	{ printf("Error (wrcoef_a) : calloc to x_new[] \n"); exit(0); }
	for(j=0;j<sx_new;j++)	x_new[j]=x[j];

	rmax=ll;
	imin=rmax-i;

	free(x);
	x=upsaconv(x_new,Lo_R,len[imin],&sx_new,sx_new);
		
	free(x_new);

	for (j=2;j<=i;j++)
	{

		if((x_new=(double *)calloc(sx_new,sizeof(double)))==NULL)
		{ printf("Error (wrcoef_a) : calloc to x_new[] \n"); exit(0); }
		for(k=0;k<sx_new;k++)	x_new[k]=x[k];	
		free(x);

		x = upsaconv(x_new,Lo_R,len[imin+j-1],&sx_new,sx_new);
		
		free(x_new);
	}
	
	return x;

}
double *appcoef(double *coef,int *length,int ll,int *sa,int level)
{
	int rmax,nmax,imax,sac,i,sdcoef,j,ssa;
	double *a,*dcoef,*acoef;

	rmax=ll;
	nmax=rmax-2;
	
	if((acoef=(double *)calloc(length[0],sizeof(double)))==NULL)
	{ printf("Error (appcoef) : calloc to a[] \n"); exit(0); }
	for(i=0;i<length[0];i++)
		acoef[i]=coef[i];
	
	ssa=length[0];
	imax=rmax+1;
	sac=ssa;
	for (i=nmax;i>=level+1;i--)
	{
		dcoef=detcoef(coef,length,ll,i,&sdcoef);

		if((a=(double *)calloc(ssa,sizeof(double)))==NULL)
		{ printf("Error (idwt) : calloc to a[] \n"); exit(0); }
		for(j=0;j<ssa;j++)	a[j]=acoef[j];
		free(acoef);
		acoef=idwt(a,dcoef,length[imax-i-1],&sac,ssa,sdcoef);
		ssa=sac;
		free(a);
		free(dcoef);
	}
	
	*sa=sac;
	
	return acoef;

}

double *idwt(double *a,double *d,int length,int *sr,int sa,int sd)
{
	int sar,sdr,i;
	double *r,*ar,*dr;

	ar=upsaconv(a,Lo_R,length,&sar,sa);
	dr=upsaconv(d,Hi_R,length,&sdr,sd);

	if((r=(double *)calloc(sar,sizeof(double)))==NULL)
	{ printf("Error (idwt) : calloc to r[] \n"); exit(0); }
	*sr=sar;
	for (i=0;i<sar;i++)	r[i]=ar[i]+dr[i];
	
	free(ar);	free(dr);
	return r;
}


double *upsaconv(double *x,double *f,int lx,int *sy,int sx)
{
	double *ux,*y1,*y2;
	int sy1,sy2,i;

	if((ux=(double *)calloc(sx*2-1,sizeof(double)))==NULL)
	{ printf("Error (upsaconv) : calloc to ux[] \n"); exit(0); }
	for (i=0;i<sx*2-1;i+=2)		ux[i]=x[i/2];
	
	y1=conv2(ux,f,sx*2-1,20);
	sy1=sx*2-1+20-1;
	y2=wkeep_r(y1,lx,sy1,&sy2);
	
	*sy=sy2;
	free(y1);
	free(ux);
	return y2;
}

double *wkeep_r(double *x,int s,int sx,int *sy)
{
	int first,last,sr,i;
	double *y,d;

	d=(double)(sx-s)/2;
	
	first = (int)(1+floor(d)); 
	last = (int)(sx-ceil(d));
	sr=last-first+1;
	
	*sy=sr;
	if((y=(double *)calloc(sr,sizeof(double)))==NULL)
	{ printf("Error (wkeep_r) : calloc to y[] \n"); exit(0); }

	for (i=0;i<sr;i++)
		y[i]=x[first-1+i];
		
		
	return y;
}

double *detcoef(double *coef,int *length, int ll,int level,int *sr)
{
	int *first,*ints,*last,i,sum,j,size_result;
	double *result;

	if((first=(int *)calloc(ll-2,sizeof(int)))==NULL)
	{ printf("Error (detcoef) : calloc to first[] \n"); exit(0); }
	if((ints=(int *)calloc(ll-2,sizeof(int)))==NULL)
	{ printf("Error (detcoef) : calloc to ints[] \n"); exit(0); }
	if((last=(int *)calloc(ll-2,sizeof(int)))==NULL)
	{ printf("Error (detcoef) : calloc to last[] \n"); exit(0); }

	for(i=ll-2-1;i>=0;i--)
	{
		sum=0;
		for (j=0;j<i+1;j++)	sum+=length[j];
		first[ll-2-1-i]=sum+1;
	}

	for(i=0;i<ll-2;i++)	ints[i]=length[ll-2-i];
	for(i=0;i<ll-2;i++)	last[i]=first[i]+ints[i]-1;
	
	size_result=last[level-1]-first[level-1]+1;
	
	if((result=(double *)calloc(size_result,sizeof(double)))==NULL)
	{ printf("Error (detcoef) : calloc to first[] \n"); exit(0); }
	*sr=size_result;	
	for(i=0;i<size_result;i++)
		result[i]=coef[first[level-1]+i-1];

	free(first);	free(last);	free(ints);
	return result;
}



//a,d=dwt(double *x)
DWTTYPE dwt(double *x,int *lx, int *ld)
{
	DWTTYPE dwt_coef;
	int i,llx,sy,sa,sd;
	int lenEXT,lenKEPT,sI;
	int *I;
	double *y,*y1,*y2,*a,*d;

	llx=*lx;
	//lenEXT = lf-1; 
	
	lenEXT=19;
	lenKEPT = llx+lf-1;
	
	I=getSymIndices(llx,lenEXT,&sI);
	
	//sI=sizeof(I)/sizeof(int);
	if((y=(double *)calloc(sI,sizeof(double)))==NULL)
	{ printf("Error (dwt) : calloc to y[] \n"); exit(0); }

	//y=x(I);
	for (i=0;i<sI;i++)
		y[i]=x[I[i]-1];

	free(I);
	
	//convdown
	
	y1=conv2(y,Lo_D,sI,20);
	y2=wkeep(y1,lenKEPT,&sy,sI+20-1);
	
	a=dyaddown(y2,sy);
	sa=sy/2;
	free(y1);	free(y2);
	//convdown
	
	y1=conv2(y,Hi_D,sI,20);
	y2=wkeep(y1,lenKEPT,&sy,sI+20-1);
	d=dyaddown(y2,sy);
	sd=sy/2;
	
	free(y1);	free(y2);
	
	free(y);

	if((dwt_coef.approximation=(double *)calloc(sa,sizeof(double)))==NULL)
	{ printf("Error (wavedec) : calloc to dwt_coef.approximation[] \n"); exit(0); }
	if((dwt_coef.details=(double *)calloc(sd,sizeof(double)))==NULL)
	{ printf("Error (wavedec) : calloc to dwt_coef.details[] \n"); exit(0); }
	
	for(i=0;i<sa;i++)	dwt_coef.approximation[i]=a[i];
	for(i=0;i<sd;i++)	dwt_coef.details[i]=d[i];
	*lx=sa;	*ld=sd;
	free(a);	free(d);

	return dwt_coef;

}

double *dyaddown(double *x,int sx)
{
	int i;
	double *y;

	
	if((y=(double *)calloc(sx/2,sizeof(double)))==NULL)
	{ printf("Error (dyaddown) : calloc to y[] \n"); exit(0); }
	
	for (i=0;i<sx/2;i++)
		y[i]=x[2*i+1];

	return y;
}

double *wkeep(double *x,int s,int *sy,int sx)
{
	double d;
	int first,last,i;
	double *y;

	d=(double)(sx-s)/2;
	
	first = (int)(1+floor(d));
	last = (int)(sx-ceil(d));
	
	//y = y(first(1):last(1));
	*sy=last-first+1;
	if((y=(double *)calloc(last-first+1,sizeof(double)))==NULL)
	{ printf("Error (conv2) : calloc to y[] \n"); exit(0); }

	
	for (i=first;i<=last;i++)
		y[i-first]=x[i-1];

	return y;
}

double *conv2(double *a, double *b,int la, int lb)
{
	int i,j;
	double *inv_b,*result;

	if((inv_b=(double *)calloc(lb,sizeof(double)))==NULL)
	{ printf("Error (conv2) : calloc to inv_b[] \n"); exit(0); }

	for (i=0;i<lb;i++)
		inv_b[i]=b[lb-i-1];

	if((result=(double *)calloc(la+lb-1,sizeof(double)))==NULL)
	{ printf("Error (conv2) : calloc to result[] \n"); exit(0); }
	
	if (la>lb)
	{
		for (i=0;i<lb;i++)
		{
			result[i]=0.;
			for (j=0;j<i+1;j++)
				result[i]+=a[j]*inv_b[lb+j-i-1];

		}

		for (i=lb;i<la;i++)
		{
			result[i]=0.;
			for (j=0;j<lb;j++)
				result[i]+=a[i+j-lb+1]*inv_b[j];
		}
		for (i=la;i<la+lb-1;i++)
		{
			result[i]=0.;
			for (j=0;j<la+lb-i-1;j++)
				result[i]+=a[i+j-lb+1]*inv_b[j];
		}
	}
	else
	{
		printf("(conv2):Did not write this part of the program because thot it is not needed\n");
		printf("%d\n%d\n",la,lb);
		exit(0);
	}



	free(inv_b);
	return result;
}


int *getSymIndices(int lx,int llf,int *sizeI)
{
	int i,sI;
	int *I,*J,*K;
	int sum;
	
	
	sI=2*llf+lx;
	*sizeI=sI;
	if((J=(int *)calloc(sI,sizeof(int)))==NULL)
	{ printf("Error (getSymIndices) : calloc to J[] \n"); exit(0); }
	if((K=(int *)calloc(sI,sizeof(int)))==NULL)
	{ printf("Error (getSymIndices) : calloc to K[] \n"); exit(0); }
	if((I=(int *)calloc(sI,sizeof(int)))==NULL)
	{ printf("Error (getSymIndices) : calloc to I[] \n"); exit(0); }


	
	//I = [lf:-1:1 , 1:lx , lx:-1:lx-lf+1];
	for (i=0;i<llf;i++)
		I[i]=llf-i;
	
	for (i=1;i<=lx;i++)
		I[i+llf-1]=i;	//till (lx+lf-1)
	
	for (i=lx-llf+1;i<=lx;i++)
		I[i+2*llf-1]=2*lx-llf+1-i;
	
	if (lx<llf)
	{
    		//K = (I<1);
		for (i=0;i<sI;i++)
			if (I[i]<1)	K[i]=1;
			else K[i]=0;

		//I(K) = 1-I(K);
		for (i=0;i<sI;i++)	
			if (K[i]==1)	I[i]=1-I[i];		

    		//J = (I>lx);
		for (i=0;i<sI;i++)
			if (I[i]>lx)	J[i]=1;
			else J[i]=0;

		sum=0;
		for (i=0;i<sI;i++)	sum+=J[i];
		
		while (sum!=0)	//while any(J)
		{
		
			//I(J) = 2*lx+1-I(J);
			for (i=0;i<sI;i++)
				if (J[i]==1)	I[i]=2*lx+1-I[i];
			for (i=0;i<sI;i++)
				if (I[i]<1)	K[i]=1;
				else K[i]=0;
		        //I(K) = 1-I(K);
			for (i=0;i<sI;i++)
				if (K[i]==1)	I[i]=1-I[i];
			//J = (I>lx);
			for (i=0;i<sI;i++)
				if (I[i]>lx)	J[i]=1;
				else J[i]=0;	
			
			sum=0;
			for (i=0;i<sI;i++)	sum+=J[i];
 		}
		
	}
	free(K);	free(J);
	return I;
}


int num_lines(FILE *in)
{
   int n=0,c;
   char string[1000];

/*   while ((c = fgetc(in)) != EOF)
     if (c == '\n') ++n;
*/
	while (fgets(string,1000,in)!=NULL)
		n++;

   rewind(in);
   return(n);
}
