#include "GetSignal.h"
/************************************************************************************************************************/
/* Purpose: 	Converts all sequences in <input fasta file> to signals with all indices listed in <input index file>. 	*/
/* 		Output direct signal to <signal file>, windowed signal to <window file> and level 2 waveform to 	*/
/*		<wave file>.												*/
/* Usage: 	GetSignal <input fasta file> <input index file> <signal file> <window file> <wave file> <WIN> <LEVEL>	*/
/*														<type>	*/
/* <type>: a for approximation, d for detail										*/
/* Output format:  <no. of seq(%d)> <no. of indices(%d)> \newline 							*/
/*			<seq. length(%d)> <data(%f) in [seq len]x[no. ofindices] \newline ...\n				*/
/* Import to matlab: Use ExtractGetSignal										*/
/************************************************************************************************************************/
void main(int argc, char **argv)
{
	int i,j,k;
	char type;
	FILE *fid;
	int index_num,seq_num;
	double **norm_aa_index, ***seq_sig, ***seq_win, ***seq_wave;
	FASTATYPE *seq;

	if (argc<6 || argc>9)
	{
		printf("Usage: GetSignal <input fasta file> <input index file> <signal file> <window file> <wave file> <WIN> <LEVEL> <type(a/d)>\n");
		printf("default: WIN=1 LEVEL=2 type=a\n");
		printf("<type>: a=approximation; d=details\n");
		printf("Purpose: Converts all sequences in <input fasta file> to signals with all indices listed in <input index file>.\n");
		printf("Output direct signal to <signal file>, windowed signal to <window file> and level 2 waveform to <wave file>.\n");
		printf("Output format: \n<no. of seq(%%d)> <no. of indices(%%d)> \n<seq. length(%%d)> <data(%%f) in [seq len]x[no. of indices] \n ...\n");
		printf("Import data to matlab: Use ExtractGetSignal.m \n");
		exit(0);
	}

	if (argc==6)	{WIN=1;		LEVEL=2;	type='a';}
	if (argc==7)	{LEVEL=2;	WIN=atoi(argv[6]);	 type='a';}
	if (argc==8)	{WIN=atoi(argv[6]);	LEVEL=atoi(argv[7]);	 type='a';}
        if (argc==9)    {WIN=atoi(argv[6]);     LEVEL=atoi(argv[7]);   type=argv[8][0];}
	printf("WIN=%d\tLEVEL=%d\ttype=%c\n",WIN,LEVEL,type);
	norm_aa_index=GetIndexKey(argv[2],&index_num);
	seq=fasta(argv[1],&seq_num);
	
	printf("Read in %d sequences\n",seq_num);
	printf("Number of indices = %d\n",index_num);

	if ((seq_sig=(double ***)calloc(seq_num,sizeof(double **)))==NULL)
	{printf("Error: calloc to seq_sig\n");	exit(0);}
	if ((seq_win=(double ***)calloc(seq_num,sizeof(double **)))==NULL)
	{printf("Error: calloc to seq_sig\n");	exit(0);}
	if ((seq_wave=(double ***)calloc(seq_num,sizeof(double **)))==NULL)
	{printf("Error: calloc to seq_sig\n");	exit(0);}

	for (i=0;i<seq_num;i++)
	{
		seq_sig[i]=GetSignal(seq[i].seq,norm_aa_index,index_num);
		seq_win[i]=SigWin(seq_sig[i],seq[i].seq_len,index_num);
		seq_wave[i]=GetWave(seq_win[i],seq[i].seq_len,index_num,type);
	}

	/********************************/
	/* Print to output file 	*/
	/********************************/
	/* Signal file */
	if ((fid=fopen(argv[3],"wt"))==NULL)
	{printf("Error opening %s\n",argv[3]);	exit(0);}
	
	fprintf(fid,"%d\t%d\n",seq_num,index_num);
	for (i=0;i<seq_num;i++)
	{
		fprintf(fid,"%d\t",seq[i].seq_len);
		for (j=0;j<index_num;j++)
		{
			for (k=0;k<seq[i].seq_len;k++)
				fprintf(fid,"%lf\t",seq_sig[i][j][k]);
		}
		fprintf(fid,"\n");
	}	

	fclose(fid);
	
	/* Windowed file */
	if ((fid=fopen(argv[4],"wt"))==NULL)
	{printf("Error opening %s\n",argv[4]);	exit(0);}
	fprintf(fid,"%d\t%d\n",seq_num,index_num);

	for (i=0;i<seq_num;i++)
	{
		fprintf(fid,"%d\t",seq[i].seq_len);
		for (j=0;j<index_num;j++)
		{
			for (k=0;k<seq[i].seq_len;k++)
				fprintf(fid,"%lf\t",seq_win[i][j][k]);
		}
		fprintf(fid,"\n");
	}	

	fclose(fid);

	/* Wave file */
	if ((fid=fopen(argv[5],"wt"))==NULL)
	{printf("Error opening %s\n",argv[5]);	exit(0);}
	fprintf(fid,"%d\t%d\n",seq_num,index_num);	

	for (i=0;i<seq_num;i++)
	{
		fprintf(fid,"%d\t",seq[i].seq_len);
		for (j=0;j<index_num;j++)
		{
			for (k=0;k<seq[i].seq_len;k++)
				fprintf(fid,"%lf\t",seq_wave[i][j][k]);
		}
		fprintf(fid,"\n");
	}	

	fclose(fid);

	/****************/
	/* Free memory	*/
	/****************/
	for (i=0;i<seq_num;i++)		{free(seq[i].name);	free(seq[i].seq);}	free(seq);
	for (i=0;i<index_num;i++)	free(norm_aa_index[i]);				free(norm_aa_index);
	for (i=0;i<seq_num;i++)		{for(j=0;j<index_num;j++)	free(seq_sig[i][j]);	free(seq_sig[i]);}	free(seq_sig);
	for (i=0;i<seq_num;i++)		{for(j=0;j<index_num;j++)	free(seq_win[i][j]);	free(seq_win[i]);}	free(seq_win);
	for (i=0;i<seq_num;i++)		{for(j=0;j<index_num;j++)	free(seq_wave[i][j]);	free(seq_wave[i]);}	free(seq_wave);
}
